/*******************************************************************************
 * Copyright 2019 Graz University of Technology
 * MOA ZS has been developed in a cooperation between EGIZ
 * and Graz University of Technology.
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.gv.egiz.moazs;

import at.gv.egiz.moazs.backend.LogResponseSink;
import at.gv.egiz.moazs.scheme.Marshaller;
import at.gv.egiz.moazs.scheme.NotificationResponse;
import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.Appender;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.slf4j.LoggerFactory;

import static at.gv.zustellung.msg.xsd.DeliveryNotificationType.deliveryNotificationTypeBuilder;
import static org.mockito.ArgumentMatchers.argThat;
import static org.mockito.Mockito.verify;

/**
 * @author Christof Rabensteiner
 *
 */
@RunWith(MockitoJUnitRunner.class)
public class LogResponseSinkTest {

    private LogResponseSink logResponseSink;

    @Mock
    private Appender appender;
    private ch.qos.logback.classic.Logger logger;

    @Before
    public void setup() {
        logResponseSink = new LogResponseSink(new Marshaller(false));
        logger = (ch.qos.logback.classic.Logger) LoggerFactory.getLogger(LogResponseSink.class);
        logger.addAppender(appender);
    }

    @Test
    public void logNotification() {
        setLogLevel(Level.INFO);
        var appDeliveryID = "log-notification-id";
        var notification = setupNotification(appDeliveryID);

        logResponseSink.log(notification)
                .thenRun(expectSubstringSentToAppender(appDeliveryID));
    }

    @Test
    public void warnWhenLogDisabled() {
        setLogLevel(Level.WARN);
        var appDeliveryID = "log-info-disabled";
        var notification = setupNotification(appDeliveryID);

        logResponseSink.log(notification)
                .thenRun(expectSubstringSentToAppender("WARN"));
    }

    private Runnable expectSubstringSentToAppender(String expectedSubstring) {
        return ()-> verify(appender).doAppend(argThat(argument -> {
            var message = ((ILoggingEvent) argument).getFormattedMessage();
            return message.contains(expectedSubstring);
        }));
    }

    private NotificationResponse setupNotification(String appDeliveryID) {

        var notification = deliveryNotificationTypeBuilder()
                .withAppDeliveryID(appDeliveryID)
                .build();

        return new NotificationResponse(notification);

    }

    private void setLogLevel(Level level) {
        logger.setLevel(level);
    }


}
